@@ -61,20 +61,33 @@ static int ssl_set_private_paths(void)
 #define ssl_set_private_paths()		(-1)
 #endif
 
+int ssl_asked_for_password;
+
+static int ssl_password_callback(char *buf, int size, int rwflag, void *userdata)
+{
+	ssl_asked_for_password = 1;
+	if (size > (int)strlen(cast_const_char ssl_options.client_cert_password))
+		size = (int)strlen(cast_const_char ssl_options.client_cert_password);
+	memcpy(buf, ssl_options.client_cert_password, size);
+	return size;
+}
+
 SSL *getSSL(void)
 {
 	if (!context) {
 		const SSL_METHOD *m;
-		unsigned char f_randfile[PATH_MAX];
 		unsigned char *os_pool;
 		int os_pool_size;
 
 #if defined(HAVE_RAND_EGD) && defined(HAVE_RAND_FILE_NAME) && defined(HAVE_RAND_LOAD_FILE) && defined(HAVE_RAND_WRITE_FILE)
-		const unsigned char *f = (const unsigned char *)RAND_file_name(cast_char f_randfile, sizeof(f_randfile));
-		if (f && RAND_egd(cast_const_char f) < 0) {
-			/* Not an EGD, so read and write to it */
-			if (RAND_load_file(cast_const_char f_randfile, -1))
-				RAND_write_file(cast_const_char f_randfile);
+		{
+			unsigned char f_randfile[PATH_MAX];
+			const unsigned char *f = (const unsigned char *)RAND_file_name(cast_char f_randfile, sizeof(f_randfile));
+			if (f && RAND_egd(cast_const_char f) < 0) {
+				/* Not an EGD, so read and write to it */
+				if (RAND_load_file(cast_const_char f_randfile, -1))
+					RAND_write_file(cast_const_char f_randfile);
+			}
 		}
 #endif
 
@@ -92,6 +105,7 @@ SSL *getSSL(void)
 		SSL_CTX_set_options(context, SSL_OP_ALL);
 		if (ssl_set_private_paths())
 			SSL_CTX_set_default_verify_paths(context);
+		SSL_CTX_set_default_passwd_cb(context, ssl_password_callback);
 
 	}
 	return SSL_new(context);
@@ -270,6 +284,25 @@ int verify_ssl_certificate(SSL *ssl, unsigned char *host)
 	return ret;
 }
 
+int verify_ssl_cipher(SSL *ssl)
+{
+	unsigned char *cipher;
+#ifdef HAVE_SSLV2_CLIENT_METHOD
+	if (SSL_get_ssl_method(ssl) == SSLv2_client_method())
+		return S_INSECURE_CIPHER;
+#endif
+#ifdef HAVE_SSLV3_CLIENT_METHOD
+	if (SSL_get_ssl_method(ssl) == SSLv3_client_method())
+		return S_INSECURE_CIPHER;
+#endif
+	if (SSL_get_cipher_bits(ssl, NULL) < 112)
+		return S_INSECURE_CIPHER;
+	cipher = cast_uchar SSL_get_cipher_name(ssl);
+	if (cipher && strstr(cast_const_char cipher, "RC4-"))
+		return S_INSECURE_CIPHER;
+	return 0;
+}
+
 #endif
 
 #else
