@@ -5,7 +5,7 @@
 
 #include "links.h"
 
-static const struct {
+static_const struct {
 	char *prot;
 	int port;
 	void (*func)(struct connection *);
@@ -53,7 +53,7 @@ static int get_prot_info(unsigned char *prot, int *port, void (**func)(struct co
 {
 	int i;
 	for (i = 0; protocols[i].prot; i++)
-		if (!strcasecmp(cast_const_char protocols[i].prot, cast_const_char prot)) {
+		if (!casestrcmp(cast_uchar protocols[i].prot, prot)) {
 			if (port) *port = protocols[i].port;
 			if (func) *func = protocols[i].func;
 			if (nc_func) *nc_func = protocols[i].nc_func;
@@ -282,23 +282,21 @@ static void translate_directories(unsigned char *url)
 		memmove(d, s, strlen(cast_const_char s) + 1);
 		return;
 	}
-	if (dsep(s[0]) && s[1] == '.' && dsep(s[2])) {
-		/**d++ = s[0];*/
-		if (s == dd && !s[3]) goto p;
+	if (dsep(s[0]) && s[1] == '.' && (dsep(s[2]) || !s[2] || end_of_dir(url, s[2]))) {
+		if (!dsep(s[2])) *d++ = *s;
 		s += 2;
 		goto r;
 	}
-	if (dsep(s[0]) && s[1] == '.' && s[2] == '.' && (dsep(s[3]) || !s[3])) {
+	if (dsep(s[0]) && s[1] == '.' && s[2] == '.' && (dsep(s[3]) || !s[3] || end_of_dir(url, s[3]))) {
 		while (d > dd) {
 			d--;
 			if (dsep(*d)) goto b;
 		}
 		b:
-		if (!s[3]) *d++ = *s;
+		if (!dsep(s[3])) *d++ = *s;
 		s += 3;
 		goto r;
 	}
-	p:
 	if ((*d++ = *s++)) goto r;
 }
 
@@ -418,6 +416,8 @@ unsigned char *join_urls(unsigned char *base, unsigned char *rel)
 		n = memacpy(base, s - base);
 		add_to_strn(&n, rel);
 		if (!parse_url(n, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)) goto return_n;
+		add_to_strn(&n, cast_uchar "/");
+		if (!parse_url(n, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)) goto return_n;
 		mem_free(n);
 	}
 	if (!casecmp(cast_uchar "proxy://", rel, 8)) goto prx;
@@ -499,124 +499,15 @@ unsigned char *translate_url(unsigned char *url, unsigned char *cwd)
 #endif
 	if (*ch != ':' || *(url + strcspn(cast_const_char url, "/@")) == '@') {
 		if (*url != '.' && *ch == '.') {
-			unsigned char *f, *e;
-			int i;
+			unsigned char *e, *f, *g;
+			int tl;
 			for (e = ch + 1; *(f = e + strcspn(cast_const_char e, ".:/")) == '.'; e = f + 1)
 				;
-			for (i = 0; i < f - e; i++) if (e[i] < '0' || e[i] > '9') goto noip;
-			goto http;
-			noip:
-			if (f - e == 2 && casecmp(e, cast_uchar "gz", 2)) {
-				http:
-				prefix = cast_uchar "http://", sl = 1;
-			} else {
-				char *tld[] = {
-					"com",
-					"edu",
-					"net",
-					"org",
-					"gov",
-					"mil",
-					"int",
-					"arpa",
-					"academy",
-					"aero",
-					"bike",
-					"biz",
-					"blue",
-					"build",
-					"builders",
-					"buzz",
-					"cab",
-					"camera",
-					"camp",
-					"careers",
-					"cat",
-					"center",
-					"ceo",
-					"christmas",
-					"clothing",
-					"club",
-					"coffee",
-					"company",
-					"computer",
-					"construction",
-					"contractors",
-					"coop",
-					"diamonds",
-					"directory",
-					"domains",
-					"education",
-					"email",
-					"enterprises",
-					"equipment",
-					"estate",
-					"eus",
-					"florist",
-					"gallery",
-					"gift",
-					"glass",
-					"graphics",
-					"guitars",
-					"guru",
-					"holdings",
-					"holiday",
-					"house",
-					"info",
-					"institute",
-					"international",
-					"jobs",
-					"kitchen",
-					"kim",
-					"land",
-					"lighting",
-					"limo",
-					"link",
-					"management",
-					"marketing",
-					"menu",
-					"mobi",
-					"museum",
-					"name",
-					"onl",
-					"photo",
-					"photography",
-					"photos",
-					"pics",
-					"pink",
-					"plumbing",
-					"post",
-					"pro",
-					"recipes",
-					"red",
-					"repair",
-					"rich",
-					"sexy",
-					"shiksha",
-					"shoes",
-					"singles",
-					"solar",
-					"solutions",
-					"support",
-					"systems",
-					"tattoo",
-					"technology",
-					"tel",
-					"tips",
-					"today",
-					"training",
-					"travel",
-					"uno",
-					"ventures",
-					"voyage",
-					"wed",
-					"wiki",
-					"xxx",
-					"onion",
-					"exit",
-					NULL };
-				for (i = 0; tld[i]; i++) if ((size_t)(f - e) == strlen(cast_const_char tld[i]) && !casecmp(cast_uchar tld[i], e, f - e)) goto http;
-			}
+			g = memacpy(e, f - e);
+			tl = is_tld(g);
+			mem_free(g);
+			if (tl)
+				http: prefix = cast_uchar "http://", sl = 1;
 		}
 		if (*ch == '@' || *ch == ':' || !cmpbeg(url, cast_uchar "ftp.")) prefix = cast_uchar "ftp://", sl = 1;
 		goto set_prefix;
@@ -708,3 +599,10 @@ void add_conv_str(unsigned char **s, int *l, unsigned char *b, int ll, int encod
 	}
 }
 
+unsigned char *display_url(struct terminal *term, unsigned char *url)
+{
+	unsigned char *u, *uu;
+	u = stracpy(url);
+	if ((uu = cast_uchar strchr(cast_const_char u, POST_CHAR))) *uu = 0;
+	return u;
+}
